Package: lme4
Version: 2.0-6
Title: Linear Mixed-Effects Models using 'Eigen' and S4
Authors@R: 
    c(person("Douglas", "Bates", role = "aut",
             comment = c(ORCID = "0000-0001-8316-9503")),
      person("Martin", "Maechler", role = "aut",
             comment = c(ORCID = "0000-0002-8685-9910")),
      person("Ben", "Bolker", role = c("cre", "aut"),
             email = "bbolker+lme4@gmail.com",
             comment = c(ORCID = "0000-0002-2127-0443")),
      person("Steven", "Walker", role = "aut",
             comment = c(ORCID = "0000-0002-4394-9078")),
      person("Anna", "Ly", role = "aut",
             comment = c(ORCID = "0000-0002-0210-0342")),
      person("Mikael", "Jagan", role = "aut",
             comment = c(ORCID = "0000-0002-3542-2938")),
      person("Rune Haubo Bojesen", "Christensen", role = "ctb",
             comment = c(ORCID = "0000-0002-4494-3399")),
      person("Henrik", "Singmann", role = "ctb",
             comment = c(ORCID = "0000-0002-4842-3657")),
      person("Bin", "Dai", role = "ctb"),
      person("Fabian", "Scheipl", role = "ctb",
             comment = c(ORCID = "0000-0001-8172-3603")),
      person("Gabor", "Grothendieck", role = "ctb"),
      person("Peter", "Green", role = "ctb",
             comment = c(ORCID = "0000-0002-0238-9852")),
      person("John", "Fox", role = "ctb"),
      person("Alexander", "Bauer", role = "ctb"),
      person("Pavel N.", "Krivitsky", role = c("ctb", "cph"),
             comment = c(ORCID = "0000-0002-9101-3362",
                         "shared copyright on simulate.formula")),
      person("Emi", "Tanaka", role = "ctb",
             comment = c(ORCID = "0000-0002-1455-259X")),
      person("Ross D.", "Boylan", role = "ctb",
             comment = c(ORCID = "0009-0003-4123-8090")),
      person("Henrik", "Bengtsson", role = "ctb",
             comment = c(ORCID = "0000-0002-9280-4430")))
Description: 
    Fit linear and generalized linear mixed-effects models.  The
    models and their components are represented using S4 classes and
    methods.  The core computational algorithms are implemented using
    the 'Eigen' C++ library for numerical linear algebra and
    'RcppEigen' "glue".
Depends: R (>= 3.6), Matrix, methods, stats
LinkingTo: Matrix (>= 1.5-0), Rcpp (>= 1.1.1-1.1), RcppEigen (>=
        0.3.3.9.4)
Imports: MASS, Rdpack, boot, graphics, grid, lattice, minqa (>=
        1.1.15), nlme (>= 3.1-123), nloptr (>= 1.0.4), parallel,
        reformulas (>= 0.4.3.1), splines, utils
Suggests: DHARMa, HSAUR3, MEMSS, car, dfoptim, future.apply, gamm4,
        ggplot2, glmmTMB, gridExtra, knitr, merDeriv, mgcv, mlmRev,
        numDeriv, optimx (>= 2013.8.6), pbkrtest, performance,
        rmarkdown, rr2, see, semEff, statmod, testthat (>= 0.8.1),
        tibble
RdMacros: Rdpack
VignetteBuilder: knitr
LazyData: yes
License: GPL (>= 2)
URL: https://github.com/lme4/lme4/
BugReports: https://github.com/lme4/lme4/issues
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-07-15 14:34:40 UTC; bolker
Author: Douglas Bates [aut] (ORCID: <https://orcid.org/0000-0001-8316-9503>),
  Martin Maechler [aut] (ORCID: <https://orcid.org/0000-0002-8685-9910>),
  Ben Bolker [cre, aut] (ORCID: <https://orcid.org/0000-0002-2127-0443>),
  Steven Walker [aut] (ORCID: <https://orcid.org/0000-0002-4394-9078>),
  Anna Ly [aut] (ORCID: <https://orcid.org/0000-0002-0210-0342>),
  Mikael Jagan [aut] (ORCID: <https://orcid.org/0000-0002-3542-2938>),
  Rune Haubo Bojesen Christensen [ctb] (ORCID:
    <https://orcid.org/0000-0002-4494-3399>),
  Henrik Singmann [ctb] (ORCID: <https://orcid.org/0000-0002-4842-3657>),
  Bin Dai [ctb],
  Fabian Scheipl [ctb] (ORCID: <https://orcid.org/0000-0001-8172-3603>),
  Gabor Grothendieck [ctb],
  Peter Green [ctb] (ORCID: <https://orcid.org/0000-0002-0238-9852>),
  John Fox [ctb],
  Alexander Bauer [ctb],
  Pavel N. Krivitsky [ctb, cph] (ORCID:
    <https://orcid.org/0000-0002-9101-3362>, shared copyright on
    simulate.formula),
  Emi Tanaka [ctb] (ORCID: <https://orcid.org/0000-0002-1455-259X>),
  Ross D. Boylan [ctb] (ORCID: <https://orcid.org/0009-0003-4123-8090>),
  Henrik Bengtsson [ctb] (ORCID: <https://orcid.org/0000-0002-9280-4430>)
Maintainer: Ben Bolker <bbolker+lme4@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-16 12:30:08 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-16 13:07:46 UTC; unix
Archs: lme4.so.dSYM
